#!/bin/bash

# BEGIN config

CABAL=`which cabal`
# END config

CABALS=""

cabal_name() {
  # $1 == cabal-file
  cat $1 | grep '^[ ]*[Nn]ame' | awk {'print $2'}
}

find_pkgs() 
{
   for cab in `ls **/*.cabal`; do
       echo -n "Compile package `cabal_name $cab`? (Y/n): "
       read CONFIRM
       case $CONFIRM in
	   n|N) 
	       echo "No"
	       ;;
	   *) 
	       echo "Yes"
	       CABALS="$cab $CABALS"
       esac
   done
}


check_errs()
{
  # Function. Parameter 1 is the return code
  # Para. 2 is text to display on failure.
  if [ "${1}" -ne "0" ]; then
    echo "${2}"
    # as a bonus, make our script exit with the right error code.
    exit ${1}
  fi
}

if [[ ${CABAL} == "" ]]; then
    echo "cannot find cabal in \${PATH}. In order to compile this software,"
    echo "please install the cabal uild system (http://www.haskell.org/cabal/)."
    exit 1
fi


find_pkgs

echo "(1/2) Installing Libraries and TcT (this may take some time)..."
$CABAL install $CABALS --force-reinstalls 2>cabal.error.log
check_errs $? "failed, see cabal.error.log."
echo "done."

echo "Creating Documentation (2/2)"
for cab in $CABALS; 
do 
 echo -n " `cabal_name $cab` ... "
 p="`dirname $cab`"
 pushd $p >/dev/zero
 $CABAL haddock 1>haddock.log 2>haddock.error.log
 if [ "$?" -ne "0" ] ; then
     echo "failed, see $p/haddock.error.log."
 else 
     echo "done."
 fi
 popd >/dev/zero
done
$CABAL install $CABALS >/dev/zero


